// NAnt - A .NET build tool
// Copyright (C) 2001 Gerry Shaw
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Gerry Shaw (gerry_shaw@yahoo.com)

namespace SourceForge.NAnt {

    using System;
    using System.Collections.Specialized;
    using System.Diagnostics;
    using System.IO;
    using System.Xml;

    [TaskName("nant")]
    public class NantTask : Task {

        [TaskAttribute("buildfile")]
        string _buildFileName = null;

        [TaskAttribute("basedir")]
        string _baseDirectory = null;

        // TODO: add support for multiple targets
        [TaskAttribute("target")]
        string _target = null;

        protected override void ExecuteTask() {
            string directory = Project.GetFullPath(_baseDirectory);
            string buildFileName = _buildFileName;
            if (buildFileName == null) {
                buildFileName = Project.FindBuildFileName(directory);
            }

            try {
                Log.WriteLine(LogPrefix + "{0} {1}", buildFileName, _target);
                Log.Indent();
                Project project = new Project();
                project.BaseDirectory = directory;
                project.BuildFileName = buildFileName;
                if (_target != null) {
                    project.BuildTargets.Add(_target);
                }
                if (!project.Run()) {
                    throw new BuildException("Nested build failed - refer to build log for exact reason.");
                }
            } finally {
                Log.Unindent();
            }
        }
    }
}